"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importStar(require("react"));
const config_1 = require("./config");
function useSWRPages(pageKey, pageFn, swrDataToOffset, deps = []) {
    const pageCountKey = `_swr_page_count_` + pageKey;
    const pageOffsetKey = `_swr_page_offset_` + pageKey;
    const [pageCount, setPageCount] = react_1.useState(config_1.cacheGet(pageCountKey) || 1);
    const [pageOffsets, setPageOffsets] = react_1.useState(config_1.cacheGet(pageOffsetKey) || [null]);
    const [pageSWRs, setPageSWRs] = react_1.useState([]);
    const pageCacheRef = react_1.useRef([]);
    const pageFnRef = react_1.useRef(pageFn);
    const emptyPageRef = react_1.useRef(false);
    // Page component (wraps `pageFn`)
    // for performance reason we need to memorize it
    const Page = react_1.useCallback(props => {
        // render the page component
        const dataList = pageFnRef.current(props);
        // if dataList is [], we can assume this page is empty
        // TODO: this API is not stable
        if (dataList && !dataList.length) {
            emptyPageRef.current = true;
        }
        else {
            emptyPageRef.current = false;
        }
        return dataList;
    }, []);
    // Doesn't have a next page
    const isReachingEnd = pageOffsets[pageCount] === null;
    const isLoadingMore = pageCount === pageOffsets.length;
    const isEmpty = isReachingEnd && pageCount === 1 && emptyPageRef.current;
    const loadMore = react_1.useCallback(() => {
        if (isLoadingMore || isReachingEnd)
            return;
        setPageCount(c => {
            config_1.cacheSet(pageCountKey, c + 1);
            return c + 1;
        });
    }, [isLoadingMore || isReachingEnd]);
    const pages = react_1.useMemo(() => {
        const getWithSWR = id => swr => {
            if (!pageSWRs[id] ||
                pageSWRs[id].data !== swr.data ||
                pageSWRs[id].error !== swr.error ||
                pageSWRs[id].revalidate !== swr.revalidate) {
                setPageSWRs(swrs => {
                    const _swrs = [...swrs];
                    _swrs[id] = swr;
                    return _swrs;
                });
                if (swr.data) {
                    // set next page's offset
                    const newPageOffset = swrDataToOffset(swr.data);
                    if (pageOffsets[id + 1] !== newPageOffset) {
                        setPageOffsets(arr => {
                            const _arr = [...arr];
                            _arr[id + 1] = newPageOffset;
                            config_1.cacheSet(pageOffsetKey, _arr);
                            return _arr;
                        });
                    }
                }
            }
            return swr;
        };
        // render each page
        const p = [];
        const pageCache = pageCacheRef.current;
        for (let i = 0; i < pageCount; ++i) {
            if (!pageCache[i] ||
                pageCache[i].offset !== pageOffsets[i] ||
                pageCache[i].pageFn !== pageFn) {
                // when props change or at init
                // render the page and cache it
                pageCache[i] = {
                    component: (react_1.default.createElement(Page, { key: `page-${pageOffsets[i]}`, offset: pageOffsets[i], withSWR: getWithSWR(i) })),
                    pageFn,
                    offset: pageOffsets[i]
                };
            }
            p.push(pageCache[i].component);
        }
        return p;
    }, [pageFn, pageCount, pageSWRs, pageOffsets, ...deps]);
    return {
        pages,
        pageCount,
        pageSWRs,
        isLoadingMore,
        isReachingEnd,
        isEmpty,
        loadMore
    };
}
exports.useSWRPages = useSWRPages;
